﻿#region Header
/*
    Little Alarm Clock
    Copyright (C) 2008 Nick H.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#endregion

#region Using
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Windows.Forms;
#endregion

namespace Little_Alarm_Clock.Logic
{
    public enum ProgramUpdateStatus { NoInit, UpToDate, NewVersion, Error };

    public class ProgramUpdater
    {
        #region Member
        public const string UPDATE_URL = "http://lilalarmclock.sourceforge.net/update.xml";
        private Version m_verCurrent = null;
        private ProgramUpdateStatus m_usStatus = ProgramUpdateStatus.NoInit;
        private Version m_verLatest = null;
        private string m_strChangeLog = string.Empty;
        private string m_strDownloadURL = string.Empty;
        #endregion


        #region Properties
        #region UpdateStatus
        public ProgramUpdateStatus UpdateStatus
        {
            get
            {
                return m_usStatus;
            }
        } 
        #endregion

        #region CurrentVersion
        public string CurrentVersion
        {
            get
            {
                return m_verCurrent.ToString();
            }
        }
        #endregion

        #region LatestVersion
        public string LatestVersion
        {
            get
            {
                string strVers = string.Empty;

                if (m_verLatest != null)
                {
                    strVers = m_verLatest.ToString();
                }
                else
                {
                    strVers = string.Empty;
                }

                return strVers;
            }
        }
        #endregion

        #region Changelog
        public string Changelog
        {
            get
            {
                return m_strChangeLog;
            }
        }
        #endregion

        #region DownloadURL
        public string DownloadURL
        {
            get
            {
                return m_strDownloadURL;
            }
        }
        #endregion 
        #endregion

        #region Init
        public void Init()
        {
            m_verCurrent = new Version(Application.ProductVersion);

            Refresh();
        } 
        #endregion

        #region Refresh
        public void Refresh()
        {
            string strLatestVersion = string.Empty;
            string strChangelog = string.Empty;
            string strDownloadURL = string.Empty;

            XmlReader xmlReader = null;
            try
            {
                xmlReader = XmlTextReader.Create(UPDATE_URL);

                while (xmlReader.Read())
                {
                    xmlReader.MoveToContent();

                    if (xmlReader.NodeType == System.Xml.XmlNodeType.Element)
                    {
                        if (xmlReader.Name.CompareTo("version") == 0)
                            strLatestVersion = xmlReader.ReadString();
                        else if (xmlReader.Name.CompareTo("changelog") == 0)
                            strChangelog = xmlReader.ReadString();
                        else if (xmlReader.Name.CompareTo("download") == 0)
                            strDownloadURL = xmlReader.ReadString();
                        else
                        {
                            //Skip
                        }
                    }
                    else
                    {
                        //Skip
                    }
                }
                m_verLatest = new Version(strLatestVersion);


                if (m_verLatest.Major > m_verCurrent.Major ||
                    m_verLatest.Minor > m_verCurrent.Minor)
                {
                    m_usStatus = ProgramUpdateStatus.NewVersion;
                }
                else
                {
                    m_usStatus = ProgramUpdateStatus.UpToDate;
                }
            }
            catch (Exception ex)
            {
                //TODO

                m_usStatus = ProgramUpdateStatus.Error;
            }
            finally
            {
                if (xmlReader != null)
                    xmlReader.Close();
            }
                        
            m_strChangeLog = strChangelog;
            m_strDownloadURL = strDownloadURL;
        } 
        #endregion
    }
}
